@@ -9,6 +9,7 @@ use Config;
 use Test::Harness::Assert;
 use Test::Harness::Iterator;
 use Test::Harness::Point;
+use Test::Harness::Results;
 
 # Flags used as return values from our methods.  Just for internal 
 # clarification.
@@ -26,9 +27,9 @@ Test::Harness::Straps - detailed analysis of test results
   my $strap = Test::Harness::Straps->new;
 
   # Various ways to interpret a test
-  my %results = $strap->analyze($name, \@test_output);
-  my %results = $strap->analyze_fh($name, $test_filehandle);
-  my %results = $strap->analyze_file($test_file);
+  my $results = $strap->analyze($name, \@test_output);
+  my $results = $strap->analyze_fh($name, $test_filehandle);
+  my $results = $strap->analyze_file($test_file);
 
   # UNIMPLEMENTED
   my %total = $strap->total_results;
@@ -93,10 +94,10 @@ sub _init {
 
 =head2 $strap->analyze( $name, \@output_lines )
 
-    my %results = $strap->analyze($name, \@test_output);
+    my $results = $strap->analyze($name, \@test_output);
 
 Analyzes the output of a single test, assigning it the given C<$name>
-for use in the total report.  Returns the C<%results> of the test.
+for use in the total report.  Returns the C<$results> of the test.
 See L<Results>.
 
 C<@test_output> should be the raw output from the test, including
@@ -117,41 +118,35 @@ sub _analyze_iterator {
 
     $self->_reset_file_state;
     $self->{file} = $name;
-    my %totals  = (
-                   max      => 0,
-                   seen     => 0,
 
-                   ok       => 0,
-                   todo     => 0,
-                   skip     => 0,
-                   bonus    => 0,
-
-                   details  => []
-                  );
+    my $results = Test::Harness::Results->new;
 
     # Set them up here so callbacks can have them.
-    $self->{totals}{$name}         = \%totals;
+    $self->{totals}{$name} = $results;
     while( defined(my $line = $it->next) ) {
-        $self->_analyze_line($line, \%totals);
+        $self->_analyze_line($line, $results);
         last if $self->{saw_bailout};
     }
 
-    $totals{skip_all} = $self->{skip_all} if defined $self->{skip_all};
+    $results->set_skip_all( $self->{skip_all} ) if defined $self->{skip_all};
 
-    my $passed = ($totals{max} == 0 && defined $totals{skip_all}) ||
-                 ($totals{max} && $totals{seen} &&
-                  $totals{max} == $totals{seen} && 
-                  $totals{max} == $totals{ok});
-    $totals{passing} = $passed ? 1 : 0;
+    my $passed =
+        (($results->max == 0) && defined $results->skip_all) ||
+        ($results->max &&
+         $results->seen &&
+         $results->max == $results->seen &&
+         $results->max == $results->ok);
 
-    return %totals;
+    $results->set_passing( $passed ? 1 : 0 );
+
+    return $results;
 }
 
 
 sub _analyze_line {
     my $self = shift;
     my $line = shift;
-    my $totals = shift;
+    my $results = shift;
 
     $self->{line}++;
 
@@ -160,7 +155,7 @@ sub _analyze_line {
     if ( $point ) {
         $linetype = 'test';
 
-        $totals->{seen}++;
+        $results->inc_seen;
         $point->set_number( $self->{'next'} ) unless $point->number;
 
         # sometimes the 'not ' and the 'ok' are on different lines,
@@ -176,14 +171,14 @@ sub _analyze_line {
         }
 
         if ( $point->is_todo ) {
-            $totals->{todo}++;
-            $totals->{bonus}++ if $point->ok;
+            $results->inc_todo;
+            $results->inc_bonus if $point->ok;
         }
         elsif ( $point->is_skip ) {
-            $totals->{skip}++;
+            $results->inc_skip;
         }
 
-        $totals->{ok}++ if $point->pass;
+        $results->inc_ok if $point->pass;
 
         if ( ($point->number > 100_000) && ($point->number > ($self->{max}||100_000)) ) {
             if ( !$self->{too_many_tests}++ ) {
@@ -201,7 +196,7 @@ sub _analyze_line {
             };
 
             assert( defined( $details->{ok} ) && defined( $details->{actual_ok} ) );
-            $totals->{details}[$point->number - 1] = $details;
+            $results->set_details( $point->number, $details );
         }
     } # test point
     elsif ( $line =~ /^not\s+$/ ) {
@@ -215,7 +210,7 @@ sub _analyze_line {
 
         $self->{saw_header}++;
 
-        $totals->{max} += $self->{max};
+        $results->inc_max( $self->{max} );
     }
     elsif ( $self->_is_bail_out($line, \$self->{bailout_reason}) ) {
         $linetype = 'bailout';
@@ -223,7 +218,8 @@ sub _analyze_line {
     }
     elsif (my $diagnostics = $self->_is_diagnostic_line( $line )) {
         $linetype = 'other';
-        my $test = $totals->{details}[-1];
+        # XXX We can throw this away, really.
+        my $test = $results->details->[-1];
         $test->{diagnostics} ||=  '';
         $test->{diagnostics}  .= $diagnostics;
     }
@@ -231,7 +227,7 @@ sub _analyze_line {
         $linetype = 'other';
     }
 
-    $self->{callback}->($self, $line, $linetype, $totals) if $self->{callback};
+    $self->{callback}->($self, $line, $linetype, $results) if $self->{callback};
 
     $self->{'next'} = $point->number + 1 if $point;
 } # _analyze_line
@@ -246,7 +242,7 @@ sub _is_diagnostic_line {
 
 =for private $strap->analyze_fh( $name, $test_filehandle )
 
-    my %results = $strap->analyze_fh($name, $test_filehandle);
+    my $results = $strap->analyze_fh($name, $test_filehandle);
 
 Like C<analyze>, but it reads from the given filehandle.
 
@@ -261,7 +257,7 @@ sub analyze_fh {
 
 =head2 $strap->analyze_file( $test_file )
 
-    my %results = $strap->analyze_file($test_file);
+    my $results = $strap->analyze_file($test_file);
 
 Like C<analyze>, but it runs the given C<$test_file> and parses its
 results.  It will also use that name for the total report.
@@ -295,20 +291,21 @@ sub analyze_file {
         return;
     }
 
-    my %results = $self->analyze_fh($file, \*FILE);
+    my $results = $self->analyze_fh($file, \*FILE);
     my $exit    = close FILE;
-    $results{'wait'} = $?;
-    if( $? && $self->{_is_vms} ) {
-        eval q{use vmsish "status"; $results{'exit'} = $?};
+
+    $results->set_wait($?);
+    if ( $? && $self->{_is_vms} ) {
+        eval q{use vmsish "status"; $results->set_exit($?); };
     }
     else {
-        $results{'exit'} = _wait2exit($?);
+        $results->set_exit( _wait2exit($?) );
     }
-    $results{passing} = 0 unless $? == 0;
+    $results->set_passing(0) unless $? == 0;
 
     $self->_restore_PERL5LIB();
 
-    return %results;
+    return $results;
 }
 
 
@@ -472,14 +469,18 @@ sub _filtered_INC {
 }
 
 
-sub _default_inc {
-    my $self = shift;
-
-    local $ENV{PERL5LIB};
-    my $perl = $self->_command;
-    my @inc =`$perl -le "print join qq[\\n], \@INC"`;
-    chomp @inc;
-    return @inc;
+{ # Without caching, _default_inc() takes a huge amount of time
+    my %cache;
+    sub _default_inc {
+        my $self = shift;
+        my $perl = $self->_command;
+        $cache{$perl} ||= [do {
+            local $ENV{PERL5LIB};
+            my @inc =`$perl -le "print join qq[\\n], \@INC"`;
+            chomp @inc;
+        }];
+        return @{$cache{$perl}};
+    }
 }
 
 
@@ -613,51 +614,6 @@ sub _reset_file_state {
     $self->{'next'}       = 1;
 }
 
-=head1 Results
-
-The C<%results> returned from C<analyze()> contain the following
-information:
-
-  passing           true if the whole test is considered a pass 
-                    (or skipped), false if its a failure
-
-  exit              the exit code of the test run, if from a file
-  wait              the wait code of the test run, if from a file
-
-  max               total tests which should have been run
-  seen              total tests actually seen
-  skip_all          if the whole test was skipped, this will 
-                      contain the reason.
-
-  ok                number of tests which passed 
-                      (including todo and skips)
-
-  todo              number of todo tests seen
-  bonus             number of todo tests which 
-                      unexpectedly passed
-
-  skip              number of tests skipped
-
-So a successful test should have max == seen == ok.
-
-
-There is one final item, the details.
-
-  details           an array ref reporting the result of 
-                    each test looks like this:
-
-    $results{details}[$test_num - 1] = 
-            { ok          => is the test considered ok?
-              actual_ok   => did it literally say 'ok'?
-              name        => name of the test (if any)
-              diagnostics => test diagnostics (if any)
-              type        => 'skip' or 'todo' (if any)
-              reason      => reason for the above (if any)
-            };
-
-Element 0 of the details is test #1.  I tried it with element 1 being
-#1 and 0 being empty, this is less awkward.
-
 =head1 EXAMPLES
 
 See F<examples/mini_harness.plx> for an example of use.
